@@ -23,27 +23,26 @@ import org.apache.samza.config.Config;
 import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.MapConfig;
 import org.apache.samza.container.LocalityManager;
-import org.apache.samza.container.TaskName;
 import org.apache.samza.coordinator.JobModelManager;
+import org.apache.samza.coordinator.JobModelManagerTestUtil;
 import org.apache.samza.coordinator.server.HttpServer;
 import org.apache.samza.coordinator.stream.messages.SetContainerHostMapping;
-import org.apache.samza.job.model.ContainerModel;
-import org.apache.samza.job.model.JobModel;
-import org.apache.samza.job.model.TaskModel;
 import org.apache.samza.metrics.MetricsRegistryMap;
+import org.apache.samza.testUtils.MockHttpServer;
 import org.eclipse.jetty.servlet.DefaultServlet;
 import org.eclipse.jetty.servlet.ServletHolder;
 import org.junit.After;
+import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
 import java.lang.reflect.Field;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.TimeUnit;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.*;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
@@ -55,17 +54,17 @@ public class TestContainerProcessManager {
 
   private Map<String, String> configVals = new HashMap<String, String>()  {
     {
-      put("yarn.container.count", "1");
-      put("systems.test-system.samza.factory", "org.apache.samza.job.yarn.MockSystemFactory");
-      put("yarn.container.memory.mb", "512");
+      put("cluster-manager.container.count", "1");
+      put("cluster-manager.container.retry.count", "1");
+      put("cluster-manager.container.retry.window.ms", "1999999999");
+      put("cluster-manager.allocator.sleep.ms", "1");
+      put("cluster-manager.container.request.timeout.ms", "2");
+      put("cluster-manager.container.memory.mb", "512");
       put("yarn.package.path", "/foo");
       put("task.inputs", "test-system.test-stream");
+      put("systems.test-system.samza.factory", "org.apache.samza.system.MockSystemFactory");
       put("systems.test-system.samza.key.serde", "org.apache.samza.serializers.JsonSerde");
       put("systems.test-system.samza.msg.serde", "org.apache.samza.serializers.JsonSerde");
-      put("yarn.container.retry.count", "1");
-      put("yarn.container.retry.window.ms", "1999999999");
-      put("yarn.allocator.sleep.ms", "1");
-      put("yarn.container.request.timeout.ms", "2");
     }
   };
   private Config config = new MapConfig(configVals);
@@ -79,7 +78,7 @@ public class TestContainerProcessManager {
   private Config getConfigWithHostAffinity() {
     Map<String, String> map = new HashMap<>();
     map.putAll(config);
-    map.put("yarn.samza.host-affinity.enabled", "true");
+    map.put("job.host-affinity.enabled", "true");
     return new MapConfig(map);
   }
 
@@ -87,30 +86,24 @@ public class TestContainerProcessManager {
 
   private SamzaApplicationState state = null;
 
-  private JobModelManager getCoordinator(int containerCount) {
-    Map<String, ContainerModel> containers = new java.util.HashMap<>();
-    for (int i = 0; i < containerCount; i++) {
-      ContainerModel container = new ContainerModel(String.valueOf(i), i, new HashMap<TaskName, TaskModel>());
-      containers.put(String.valueOf(i), container);
-    }
+  private JobModelManager getJobModelManagerWithHostAffinity(int containerCount) {
     Map<String, Map<String, String>> localityMap = new HashMap<>();
     localityMap.put("0", new HashMap<String, String>() { {
         put(SetContainerHostMapping.HOST_KEY, "abc");
-      }
-    });
+      } });
     LocalityManager mockLocalityManager = mock(LocalityManager.class);
     when(mockLocalityManager.readContainerLocality()).thenReturn(localityMap);
 
-    JobModel jobModel = new JobModel(getConfig(), containers, mockLocalityManager);
-    JobModelManager.jobModelRef().getAndSet(jobModel);
+    return JobModelManagerTestUtil.getJobModelManagerWithLocalityManager(getConfig(), containerCount, mockLocalityManager, this.server);
+  }
 
-    return new JobModelManager(jobModel, this.server, null);
+  private JobModelManager getJobModelManagerWithoutHostAffinity(int containerCount) {
+    return JobModelManagerTestUtil.getJobModelManager(getConfig(), containerCount, this.server);
   }
 
   @Before
   public void setup() throws Exception {
     server = new MockHttpServer("/", 7777, null, new ServletHolder(DefaultServlet.class));
-    state = new SamzaApplicationState(getCoordinator(1));
   }
 
   private Field getPrivateFieldFromTaskManager(String fieldName, ContainerProcessManager object) throws Exception {
@@ -124,9 +117,10 @@ public class TestContainerProcessManager {
   public void testContainerProcessManager() throws Exception {
     Map<String, String> conf = new HashMap<>();
     conf.putAll(getConfig());
-    conf.put("yarn.container.memory.mb", "500");
-    conf.put("yarn.container.cpu.cores", "5");
+    conf.put("cluster-manager.container.memory.mb", "500");
+    conf.put("cluster-manager.container.cpu.cores", "5");
 
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
     ContainerProcessManager taskManager = new ContainerProcessManager(
         new MapConfig(conf),
         state,
@@ -143,9 +137,10 @@ public class TestContainerProcessManager {
 
     conf.clear();
     conf.putAll(getConfigWithHostAffinity());
-    conf.put("yarn.container.memory.mb", "500");
-    conf.put("yarn.container.cpu.cores", "5");
+    conf.put("cluster-manager.container.memory.mb", "500");
+    conf.put("cluster-manager.container.cpu.cores", "5");
 
+    state = new SamzaApplicationState(getJobModelManagerWithHostAffinity(1));
     taskManager = new ContainerProcessManager(
         new MapConfig(conf),
         state,
@@ -164,6 +159,8 @@ public class TestContainerProcessManager {
   @Test
   public void testOnInit() throws Exception {
     Config conf = getConfig();
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
     ContainerProcessManager taskManager = new ContainerProcessManager(
         new MapConfig(conf),
         state,
@@ -177,15 +174,19 @@ public class TestContainerProcessManager {
         state);
 
     getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
-
+    CountDownLatch latch = new CountDownLatch(1);
     getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, new Thread() {
       public void run() {
         isRunning = true;
+        latch.countDown();
       }
     });
 
     taskManager.start();
-    Thread.sleep(1000);
+
+    if (!latch.await(2, TimeUnit.SECONDS)) {
+      Assert.fail("timed out waiting for the latch to expire");
+    }
 
     // Verify Allocator thread has started running
     assertTrue(isRunning);
@@ -200,6 +201,8 @@ public class TestContainerProcessManager {
   @Test
   public void testOnShutdown() throws Exception {
     Config conf = getConfig();
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
     ContainerProcessManager taskManager =  new ContainerProcessManager(
         new MapConfig(conf),
         state,
@@ -208,38 +211,56 @@ public class TestContainerProcessManager {
     );
     taskManager.start();
 
-    Thread.sleep(100);
-
     Thread allocatorThread = (Thread) getPrivateFieldFromTaskManager("allocatorThread", taskManager).get(taskManager);
     assertTrue(allocatorThread.isAlive());
 
     taskManager.stop();
 
-    Thread.sleep(100);
     assertFalse(allocatorThread.isAlive());
-
   }
 
   /**
    * Test Task Manager should stop when all containers finish
    */
   @Test
-  public void testTaskManagerShouldStopWhenContainersFinish() {
+  public void testTaskManagerShouldStopWhenContainersFinish() throws Exception {
     Config conf = getConfig();
-    ContainerProcessManager taskManager =  new ContainerProcessManager(
-        new MapConfig(conf),
-        state,
-        new MetricsRegistryMap(),
-        manager
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
+    ContainerProcessManager taskManager = new ContainerProcessManager(
+            new MapConfig(conf),
+            state,
+            new MetricsRegistryMap(),
+            manager
     );
 
+    MockContainerAllocator allocator = new MockContainerAllocator(
+            manager,
+            conf,
+            state);
+
+    getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
+
+    Thread thread = new Thread(allocator);
+    getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
+
+    // start triggers a request
     taskManager.start();
 
     assertFalse(taskManager.shouldShutdown());
+    assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
-    taskManager.onResourceCompleted(new SamzaResourceStatus("123", "diagnostics", SamzaResourceStatus.SUCCESS));
+    SamzaResource container = new SamzaResource(1, 1024, "abc", "id0");
+    taskManager.onResourceAllocated(container);
 
+    // Allow container to run and update state
 
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    assertFalse(taskManager.shouldShutdown());
+
+    taskManager.onResourceCompleted(new SamzaResourceStatus("id0", "diagnostics", SamzaResourceStatus.SUCCESS));
     assertTrue(taskManager.shouldShutdown());
   }
 
@@ -251,6 +272,7 @@ public class TestContainerProcessManager {
   @Test
   public void testNewContainerRequestedOnFailureWithUnknownCode() throws Exception {
     Config conf = getConfig();
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
     ContainerProcessManager taskManager = new ContainerProcessManager(
         new MapConfig(conf),
@@ -280,7 +302,9 @@ public class TestContainerProcessManager {
     taskManager.onResourceAllocated(container);
 
     // Allow container to run and update state
-    Thread.sleep(300);
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
 
     // Create first container failure
     taskManager.onResourceCompleted(new SamzaResourceStatus(container.getResourceID(), "diagnostics", 1));
@@ -298,7 +322,9 @@ public class TestContainerProcessManager {
     taskManager.onResourceAllocated(container);
 
     // Allow container to run and update state
-    Thread.sleep(1000);
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
 
     assertTrue(state.jobHealthy.get());
 
@@ -317,6 +343,117 @@ public class TestContainerProcessManager {
     taskManager.stop();
   }
 
+  @Test
+  public void testInvalidNotificationsAreIgnored() throws Exception {
+    Config conf = getConfig();
+
+    Map<String, String> config = new HashMap<>();
+    config.putAll(getConfig());
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
+    ContainerProcessManager taskManager = new ContainerProcessManager(
+            new MapConfig(conf),
+            state,
+            new MetricsRegistryMap(),
+            manager
+    );
+
+    MockContainerAllocator allocator = new MockContainerAllocator(
+            manager,
+            conf,
+            state);
+    getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
+
+    Thread thread = new Thread(allocator);
+    getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
+
+    // Start the task manager
+    taskManager.start();
+
+    SamzaResource container = new SamzaResource(1, 1000, "abc", "id1");
+    taskManager.onResourceAllocated(container);
+
+    // Allow container to run and update state
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+
+    // Create container failure - with ContainerExitStatus.DISKS_FAILED
+    taskManager.onResourceCompleted(new SamzaResourceStatus("invalidContainerID", "Disk failure", SamzaResourceStatus.DISK_FAIL));
+
+    // The above failure should not trigger any container requests, since it is for an invalid container ID
+    assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
+    assertFalse(taskManager.shouldShutdown());
+    assertTrue(state.jobHealthy.get());
+    assertEquals(state.redundantNotifications.get(), 1);
+  }
+
+  @Test
+  public void testDuplicateNotificationsDoNotAffectJobHealth() throws Exception {
+    Config conf = getConfig();
+
+    Map<String, String> config = new HashMap<>();
+    config.putAll(getConfig());
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
+    ContainerProcessManager taskManager = new ContainerProcessManager(
+            new MapConfig(conf),
+            state,
+            new MetricsRegistryMap(),
+            manager
+    );
+
+    MockContainerAllocator allocator = new MockContainerAllocator(
+            manager,
+            conf,
+            state);
+    getPrivateFieldFromTaskManager("containerAllocator", taskManager).set(taskManager, allocator);
+
+    Thread thread = new Thread(allocator);
+    getPrivateFieldFromTaskManager("allocatorThread", taskManager).set(taskManager, thread);
+
+    // Start the task manager
+    taskManager.start();
+    assertFalse(taskManager.shouldShutdown());
+    assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
+
+    SamzaResource container1 = new SamzaResource(1, 1000, "abc", "id1");
+    taskManager.onResourceAllocated(container1);
+
+    // Allow container to run and update state
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
+
+    // Create container failure - with ContainerExitStatus.DISKS_FAILED
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "Disk failure", SamzaResourceStatus.DISK_FAIL));
+
+    // The above failure should trigger a container request
+    assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
+    assertFalse(taskManager.shouldShutdown());
+    assertFalse(state.jobHealthy.get());
+    assertEquals(2, manager.resourceRequests.size());
+    assertEquals(0, manager.releasedResources.size());
+    assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
+
+    SamzaResource container2 = new SamzaResource(1, 1000, "abc", "id2");
+    taskManager.onResourceAllocated(container2);
+
+    // Allow container to run and update state
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    assertTrue(state.jobHealthy.get());
+
+    // Simulate a duplicate notification for container 1 with a different exit code
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "Disk failure", SamzaResourceStatus.PREEMPTED));
+    // assert that a duplicate notification does not change metrics (including job health)
+    assertEquals(state.redundantNotifications.get(), 1);
+    assertEquals(2, manager.resourceRequests.size());
+    assertEquals(0, manager.releasedResources.size());
+    assertTrue(state.jobHealthy.get());
+  }
 
   /**
    * Test AM requests a new container when a task fails
@@ -328,7 +465,7 @@ public class TestContainerProcessManager {
 
     Map<String, String> config = new HashMap<>();
     config.putAll(getConfig());
-    config.remove("yarn.container.retry.count");
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
 
     ContainerProcessManager taskManager = new ContainerProcessManager(
         new MapConfig(conf),
@@ -351,14 +488,17 @@ public class TestContainerProcessManager {
     assertFalse(taskManager.shouldShutdown());
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
 
-    SamzaResource container = new SamzaResource(1, 1000, "abc", "id1");
-    taskManager.onResourceAllocated(container);
+    SamzaResource container1 = new SamzaResource(1, 1000, "abc", "id1");
+    taskManager.onResourceAllocated(container1);
 
     // Allow container to run and update state
-    Thread.sleep(300);
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+    assertEquals(0, allocator.getContainerRequestState().numPendingRequests());
 
     // Create container failure - with ContainerExitStatus.DISKS_FAILED
-    taskManager.onResourceCompleted(new SamzaResourceStatus(container.getResourceID(), "Disk failure", SamzaResourceStatus.DISK_FAIL));
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container1.getResourceID(), "Disk failure", SamzaResourceStatus.DISK_FAIL));
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
@@ -368,21 +508,37 @@ public class TestContainerProcessManager {
     assertEquals(0, manager.releasedResources.size());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
 
+    SamzaResource container2 = new SamzaResource(1, 1000, "abc", "id2");
+    taskManager.onResourceAllocated(container2);
+
+    // Allow container to run and update state
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
+
     // Create container failure - with ContainerExitStatus.PREEMPTED
-    taskManager.onResourceCompleted(new SamzaResourceStatus(container.getResourceID(), "Preemption",  SamzaResourceStatus.PREEMPTED));
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container2.getResourceID(), "Preemption",  SamzaResourceStatus.PREEMPTED));
+    assertEquals(3, manager.resourceRequests.size());
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
     assertEquals(ResourceRequestState.ANY_HOST, allocator.getContainerRequestState().peekPendingRequest().getPreferredHost());
+    SamzaResource container3 = new SamzaResource(1, 1000, "abc", "id3");
+    taskManager.onResourceAllocated(container3);
+
+    // Allow container to run and update state
+    if (!allocator.awaitContainersStart(1, 2, TimeUnit.SECONDS)) {
+      fail("timed out waiting for the containers to start");
+    }
 
     // Create container failure - with ContainerExitStatus.ABORTED
-    taskManager.onResourceCompleted(new SamzaResourceStatus(container.getResourceID(), "Aborted", SamzaResourceStatus.ABORTED));
+    taskManager.onResourceCompleted(new SamzaResourceStatus(container3.getResourceID(), "Aborted", SamzaResourceStatus.ABORTED));
 
     // The above failure should trigger a container request
     assertEquals(1, allocator.getContainerRequestState().numPendingRequests());
-    assertEquals(2, manager.resourceRequests.size());
+    assertEquals(4, manager.resourceRequests.size());
     assertEquals(0, manager.releasedResources.size());
     assertFalse(taskManager.shouldShutdown());
     assertFalse(state.jobHealthy.get());
@@ -393,8 +549,11 @@ public class TestContainerProcessManager {
 
   @Test
   public void testAppMasterWithFwk() {
+    Config conf = getConfig();
+    state = new SamzaApplicationState(getJobModelManagerWithoutHostAffinity(1));
+
     ContainerProcessManager taskManager = new ContainerProcessManager(
-        new MapConfig(config),
+        new MapConfig(conf),
         state,
         new MetricsRegistryMap(),
         manager
